{% for tag, operations in handler.get_operations().items() %}

## {{tag or "Endpoints"}}

{% for path, definition in operations %}
{%- for http_method, operation in definition.items() %}

### {{http_method.upper()}} {{path | safe}}
{% if "summary" in operation -%}
{{operation.summary | wordwrap(80)}}
{%- endif -%}

{%- if operation.description and operation.summary != operation.description %}

**{{texts.description.title()}}**
{{operation.description | wordwrap(80)}}

{%- endif -%}

{%- with parameters = handler.get_parameters(operation) -%}
{%- if parameters %}
{% include "partial/request-parameters.html" %}
{%- endif -%}
{%- endwith -%}

{%- if operation.requestBody %}

{% include "partial/request-body.html" %}
{%- endif %}

{%- if operation.responses %}
{%- include "partial/request-responses.html" %}
{%- endif %}
{%- endfor -%}
{%- endfor -%}
{%- endfor -%}
