@startuml "Components schemas"
/'
This diagram has been generated by essentials-openapi
https://github.com/Neoteroi/essentials-openapi

Most likely, it is not desirable to edit this file by hand!
'/
top to bottom direction
hide empty members

{% for name, definition in handler.get_schemas() %}
class {{name}} {
    {%- if definition.type == "object" -%}
    {%- for name, schema in handler.get_properties(definition) %}
    {{name}}: {% include "partial/schema-repr.html" %}
    {%- endfor -%}
    {%- else -%}
    {%- with schema = definition %}
    {% include "partial/schema-repr.html" -%}
    {%- endwith -%}
    {%- endif %}
}
{% endfor %}

{% for type_a, type_b in handler.iter_schemas_bindings() %}
{{type_a}} --> {{type_b}}
{% endfor %}
@enduml
